;	BPAINT Regular map loading and Showing
;	By Angus Arthur in 2002, released with BPAINT.
;
;	Here is a program that loads and displays a "regular" map.
;	A regular map is one that is stored as a list of "map cells", each of which 
;	containing one "block".  This list of cells is then arranged in a grid.
;
;	This code is just an example of how to load and display such a map.
;	This is the structure of the file, so you can do what you want!:
;
;	Line1:  String: "Regular Map" 
;	Line2:  String: MAPNAME$	-there if you need it
;	Line3:	Integer: Mapwidthvariable	-required by BPaint, ignore it!
;	Line4:	Integer: Mapheightvariable	-required by BPaint, ignore it!
;	Line4:	Integer: MAPWIDTH	-width of the map, in cells
;	Line5:	Integer: MAPHEIGHT	-height of the map, in cells
;	Line6:	Integer: CELLWIDTH	-width of each cell, in pixels
;	Line7:	Integer: CELLHEIGHT	-height of each cell, in pixels
;	Line8:	Integer:	First cell's image reference
;	Line9:	Integer:	First cell's strip reference
;	Lint10+: Successive cell details (Lines 8,9), reading Left to right, top to bottom.
;
;	In this example, maps are of a maximum size of 1000*1000, just change the relevant bits if you 
;	need to.
;
;	This example makes space to load more than one strip, but uses only one.
;	If you take a look at the code, you should be able to see that it will support the 
;	loading of maps that use multiple strips.
;


;	Type of variable to store general details about each map
Type mapinfo			
	Field gridw,gridh
	Field width,height
End Type

;	Type of variable to describe each cell in a map
Type mapcomponent	
	Field image
	Field strip
End Type

;	Dimension variable into which to load block strips
Dim blocks(5)

;	Dimension variable to store description of map
;	First Dimension refers to a map number, in this case, up to 5 maps can be loaded
;	Second Dimension refers to the column numbers, or X position of each cell
;	Third Dimension refers to the row numbers, or the Y position of each cell
;	so any block is referred to by its map number, and its X and Y position in the array
Dim map.mapcomponent(5,1000,1000)

;	Dimension variable that stores details of each map
;	Dimension refers to Map number, again up to five, use the same number to refer to 
;	an individual map cell, in the above variable
Dim mapinfo.mapinfo(5)
For n=1 To 5
	mapinfo.mapinfo(n)=New mapinfo
Next

;Globals
Global Swidth=640,Sheight=480

; Open the screen and load our files
Graphics Swidth,Sheight,32
SetBuffer BackBuffer()

;	Here I load my block list into the first part of my blocks variable
;	Further lists could be loaded into successive parts.
blocks(1)=LoadAnimImage ("blocks.bmp",32,32,0,5)

;	Loadregmap loads the map of the given name into the "map slot" referred To by the First
;	parameter.  So here: Load map 1, with "Regmap.map"
Check=loadregmap(1,"regmap.map")

If Check>1	;the loadregmap function returns 1 when everythings OK, and other numbers depending
	DebugLog Check	;on any problem that there may be.
	Stop
End If

;	These variables define the display position
ScreenX=0 : ScreenY=0

Repeat
	Flip
	Cls
	ScreenX=ScreenX+MouseXSpeed()
	ScreenY=ScreenY+MouseYSpeed()
	MoveMouse 100,100
	drawregmap(1,ScreenX,ScreenY,0,0,Swidth,Sheight)
	Color 255,255,255:Text 10,10,"Regular Map:Move around with the mouse, push ESC to exit"
Until KeyDown(1)

EndGraphics
End


;	This function displays a map, referred to by it's number
;	It'll be drawn within the box defined by the "disp" parameters.
;	It'll be drawn from the position defined by the "pos" parameters.
Function drawregmap(num,xpos,ypos,dispx,dispy,dispw,disph)
	Viewport dispx,dispy,dispw,disph
	blockw=mapinfo(num)\gridw
	blockh=mapinfo(num)\gridh
	dispblockw=(dispw/blockw)+1
	dispblockh=(disph/blockh)+1
	xblockoff=(xpos/blockw)
	yblockoff=(ypos/blockh)
	xoff=(xblockoff+1)*blockw-xpos
	yoff=(yblockoff+1)*blockh-ypos
	For y=0 To dispblockh
		For x=0 To dispblockw
			cbx=x+xblockoff
			cby=y+yblockoff
			If cbx>=0 And cby>=0
				If cbx<mapinfo(num)\width And cby<mapinfo(num)\height
					st=map(num,cbx,cby)\strip
					im=map(num,cbx,cby)\image
					cx=(dispx+((x-1)*blockw))+xoff
					cy=(dispy+((y-1)*blockh))+yoff
					If st>0
						DrawImage blocks(st),cx,cy,im
					End If
				End If
			End If
		Next
	Next
	Viewport 0,0,Swidth,Sheight
End Function


;	This function loads the map.
Function loadregmap(num,f$)
	nmap=OpenFile(f$)
	If nmap=0
		Return 4
	Else
		row$=ReadLine$(nmap)
		If row$="Regular Map"
			Nowt$=ReadLine$(nmap)
			Nowt=ReadInt(nmap)
			Nowt=ReadInt(nmap)
			mapw=ReadInt(nmap)
			maph=ReadInt(nmap)
			gridw=ReadInt(nmap)
			gridh=ReadInt(nmap)
			mapinfo(num)\gridw=gridw
			mapinfo(num)\gridh=gridh
			mapinfo(num)\width=mapw
			mapinfo(num)\height=maph
			If mapw<1000 And maph<1000
				For y=0 To maph
					For x=0 To mapw
						map.mapcomponent(num,x,y)=New mapcomponent
					Next
				Next	
				For y=0 To maph
					For x=0 To mapw
						imge=ReadInt(nmap)
						strip=ReadInt(nmap)
						map(num,x,y)\image=imge
						map(num,x,y)\strip=strip
					Next
				Next
			Else
				CloseFile nmap
				Return 3
			End If
		Else
			CloseFile nmap
			Return 2
		End If
		CloseFile nmap
		Return 1
	End If
End Function













