;	BPAINT Irregular map loading and Showing
;	By Angus Arthur in 2002, released with BPAINT.
;
;	Here is a program that loads and displays an "Irregular" map.
;	An Irregular map is stored as a list of large cells, each of which
;	holds a list of "blocks" along with their image number, strip number
;	and position within the cell.
;	BPaint makes cells of 640*480.
;
;	This code is just an example of how to load and display such a map.
;	This is the structure of the file, so you cna do what you want!:
;
;	Line1:  String: "Regular Map" 
;	Line2:  String: MAPNAME$	-there if you need it
;	Line3:	Integer: MAPWIDTH	-width of map in cells
;	Line4:	Integer: MAPHEIGHT	-height of map in cells
;	Line5:	Integer: CELLWIDTH	-width of each cell in pixels (will load 640)
;	Line6:	Integer: CELLHEIGHT	-height of each cell in pixels (will load 480)
;	Line7:	Integer: gridw -for BPaint only, ignore it!
;	Line8:	Integer: gridh -for BPaint only, ignore it!
;	Line9:	Integer:number of pieces in the first cell
;	Line10: Integer: IMAGE - number of the image of the first piece in the first cell
;	Line11:	Integer: STRIP - number of the strip of the first piece in the first cell
;	Line12: Integer: X -Xpos of first piece in first cell
;	Line13:	Integer: Y -Ypos of the first piece in the first cell
;	Line14+:  Successive pieces in the first cell, until all are read.
;          Then from Line9 for successive cells, reading left to right, top to bottom
;
;	This example makes space to load more than one strip, but uses only one.
;	If you take a look at the code, you should be able to see that it will support the 
;	loading of maps that use multiple strips.
;


;	Type of variable to store general details about each map
;	including the size of each cell in pixels and the side of the map in cells
Type mapinfo			
	Field cellwidth,cellheight
	Field width,height
End Type

;	Type of variable to describe each cell in a map
;	in this case, each cell can contain up to 1000 blocks
Type piecelist
	Field numpieces
	Field image[1000]
	Field strip[1000]
	Field x[1000]
	Field y[1000]
End Type

;	Dimension variable into which to load block strips
Dim blocks(5)

;	Dimension variable holding the details of the maps
;	First Dimension refers to the map number
;	Second Dimension refers to the colomn number, or Cell X position
;	Third Dimension refers to the Row number, or Cell Y position
;	Each component of this varable is a list of blocks in the cell
;	as each cell is 640*480, in this case maps can be up to 40*40 screens,
;	which is currently the maximum that BPaint can make.
Dim map.piecelist(5,40,40)

;	Dimension variable that stores details of each map
;	Dimension refers to Map number, again up to five, use the same number to refer to 
;	an individual map cell, in the above variable
Dim mapinfo.mapinfo(5)
For n=1 To 5
	mapinfo.mapinfo(n)=New mapinfo
Next

;	Globals
Global Swidth=640,Sheight=480

;	Open the screen and load our files
Graphics Swidth,Sheight,32
SetBuffer BackBuffer()

;	Here I load my block list into the first part of my blocks variable
;	Further lists could be loaded into successive parts.
blocks(1)=LoadAnimImage ("blocks.bmp",32,32,0,5)

;	Loadregmap loads the map of the given name into the "map slot" referred to by the first
;	parameter.  So here: Load map 1, with "Regmap.map"
Check=loadirregmap(1,"irregmap.map")

If Check>1	;the loadirregmap function returns 1 when everythings OK, and other numbers depending
	DebugLog Check	;on any problem that there may be.
	Stop
End If

;	These variables define the display position
ScreenX=0 : ScreenY=0

Repeat
	Flip
	Cls
	ScreenX=ScreenX+MouseXSpeed()
	ScreenY=ScreenY+MouseYSpeed()
	MoveMouse 100,100
	drawirregmap(1,ScreenX,ScreenY,0,0,Swidth,Sheight)
	Color 255,255,255:Text 10,10,"Irregular Map:Move around with the mouse, push ESC to exit"
Until KeyDown(1)

EndGraphics
End


;	This function displays a map, referred to by it's number
;	It'll be drawn within the box defined by the "disp" parameters.
;	It'll be drawn from the position defined by the "pos" parameters.
Function drawirregmap(mapnum,xpos,ypos,dispx,dispy,dispw,disph)
	Viewport dispx,dispy,dispw,disph
	cw=mapinfo(mapnum)\Cellwidth
	ch=mapinfo(mapnum)\Cellheight
	w=mapinfo(mapnum)\width
	h=mapinfo(mapnum)\height
	For y=1 To h
		For x=1 To w
			If RectsOverlap((x-1)*cw,(y-1)*ch,cw,ch,xpos,ypos,dispw,disph)
				cxo=dispx-(xpos-(x-1)*cw)
				cyo=dispy-(ypos-(y-1)*ch)
				If map(mapnum,x,y)\numpieces>0
					For n=1 To map(mapnum,x,y)\numpieces
						px=cxo+map(mapnum,x,y)\x[n]
						py=cyo+map(mapnum,x,y)\y[n]
						im=map(mapnum,x,y)\image[n]
						st=map(mapnum,x,y)\strip[n]
						DrawImage blocks(st),px,py,im
					Next
				End If
			End If
		Next
	Next
	Viewport 0,0,Swidth,Sheight
End Function

;	This function loads the map
Function loadirregmap(num,f$)
	nmap=OpenFile(d$+f$)
	If nmap=0
		Return 4
	Else
		row$=ReadLine$(nmap)
		If row$="Irregular Map"
			row$=ReadLine$(nmap)
			mw=ReadInt(nmap)
			mh=ReadInt(nmap)
			cw=ReadInt(nmap)
			ch=ReadInt(nmap)
			n=ReadInt(nmap)
			n=ReadInt(nmap)
			mapinfo(num)\width=mw
			mapinfo(num)\height=mh
			mapinfo(num)\cellwidth=cw
			mapinfo(num)\cellheight=ch
			If mw<40 And mh<20
				For y=1 To mh
					For x=1 To mw
						cp=ReadInt(nmap)
						map.piecelist(num,x,y)=New piecelist
						If cp>0
							For n=1 To cp
								map(num,x,y)\image[n]=ReadInt(nmap)
								map(num,x,y)\strip[n]=ReadInt(nmap)
								map(num,x,y)\x[n]=ReadInt(nmap)
								map(num,x,y)\y[n]=ReadInt(nmap)
							Next
						End If
						map(num,x,y)\numpieces=cp
					Next
				Next
			Else
				CloseFile nmap
				Return 3
			End If
		Else
			CloseFile nmap
			Return 2
		End If
		CloseFile nmap
		Return 1
	End If
End Function


















